from pathlib import Path
from pathspec import PathSpec
from pathspec.patterns import GitWildMatchPattern
import toml


def read_text(path: Path) -> str:
    """Assuming all non-textual files are in the pyproject.toml ignore list"""
    try:
        with open(path, "r") as file:
            return file.read()

    except UnicodeDecodeError:
        with open(path, "r", encoding="utf-8", errors="replace") as file:
            return file.read()
    except Exception as e:
        raise e


def collect_text_from_files(dir_path: Path, pathspec: PathSpec) -> dict:
    """recursively collect text from all files in a directory"""
    result = {}
    for path in dir_path.iterdir():
        if path.is_file():
            if not pathspec.match_file(str(path)):
                result[path] = read_text(path)
        elif path.is_dir():
            result.update(collect_text_from_files(path, pathspec))
    return result


def create_default_config(toml_path):
    """Create a pyproject.toml with default ignore patterns"""
    print("Creating default pyproject.toml with entry for turbo_docs")
    configs = toml.load(toml_path) if toml_path.exists() else {}

    if "tool" not in configs:
        configs["tool"] = {}
    configs["tool"]["turbo_docs"] = {
        "ignore": [
            "pyproject.toml",
            "__pycache__",
            "venv",
            "build",
            "dist",
            "*.egg-info",
            ".git",
            ".env",
        ]
    }

    with open(toml_path, "w") as toml_file:
        toml.dump(configs, toml_file)


def get_repo_text_dict():
    toml_path = Path("pyproject.toml")

    if (
        not toml_path.exists()
        or "tool" not in toml.load(toml_path)
        or "turbo_docs" not in toml.load(toml_path)["tool"]
    ):
        create_default_config(toml_path)

    config = toml.load(toml_path)["tool"]["turbo_docs"]

    ignored_patterns = config.get("ignore", [])
    print(f"Ignoring the following patterns: {ignored_patterns}")

    pathspec = PathSpec.from_lines(GitWildMatchPattern, ignored_patterns)
    return collect_text_from_files(Path("."), pathspec)


def convert_dict_to_string(file_dict: dict):
    text = ""
    for path, content in file_dict.items():
        text += f"{path}:\n\n{content}\n\n"
    return text



TEMPLATE = """
"You are an expert software developement assistant. Write a README.md for the following repo:
{repo}

- Imagine this repo were trending on GitHub. What sort of information is required in a readme of this caliber?
- Write from the perspective of the user and from a contributer. Curate a best in class user experience by providing ample detail, context, and examples.
- emoji are encouraged. One for each section at minimum.
"""


def readme(repo, template=TEMPLATE):
    """
    Chose between GPT-3.5 Turbo and GPT-4, allow for template override, and
    generate a README.md file for the current repo.
    """
    from turbo_docs.utils import openai_api

    readme = "README.md"
    prompt = TEMPLATE.format(repo=repo)
    response = openai_api.gpt_completion(prompt)

    with open(readme, "w", encoding="utf-8") as readme_file:
        readme_file.write(response)



from pathlib import Path

import click
import pyperclip
import tiktoken

from turbo_docs.commands import readme as readme_module
from turbo_docs.utils import cli_options, directory


def num_tokens_from_string(string: str, encoding_name: str = "cl100k_base") -> int:
    """Returns the number of tokens in a text string."""
    encoding = tiktoken.get_encoding(encoding_name)
    num_tokens = len(encoding.encode(string, allowed_special="all"))
    return num_tokens


@click.command()
@cli_options.copy
@cli_options.readme
def driver(
    copy: bool,
    readme: bool,
) -> None:
    """
    Pull text from all files in the current directory and apply the following commands:

    'turbo_docs --copy' copies the text to clipboard - Useful for wokring with ChatGPT
    'turbo_docs --readme' generates a README.md file - Useful for keeping documentation up to date
    """
    dir_text_dict = directory.get_repo_text_dict()

    if readme:
        if Path("README.md") in dir_text_dict:
            del dir_text_dict[Path("README.md")]
        dir_text_str = directory.convert_dict_to_string(dir_text_dict)
        print(dir_text_str)
        readme_module.readme(dir_text_str)
        print("Generated README.md")

    if copy:
        dir_text_str = directory.convert_dict_to_string(dir_text_dict)
        num_tokens = num_tokens_from_string(dir_text_str)
        pyperclip.copy(dir_text_str)
        print(f"Directory text copied to clipboard containing {num_tokens} tokens")



from typing import Callable

import click


def copy(func: Callable) -> Callable:
    return click.option(
        "--copy",
        default=False,
        is_flag=True,
        help="Copy the directory text to clipboard",
    )(func)


def readme(func: Callable) -> Callable:
    return click.option(
        "--readme", default=False, is_flag=True, help="Generate README.md file"
    )(func)




import os

if not os.environ.get("OPENAI_API_KEY"):
    print(
        "OpenAI API key is not set. Please set it as an environment variable (export OPENAI_API_KEY=<your_api_key>) or enter it below."
    )
    print(
        "If you have not done so already, create an OpenAI account at https://platform.openai.com/overview."
    )
    os.environ["OPENAI_API_KEY"] = input("Secret key: ")

from openai import OpenAI


def gpt_completion(prompt, model="gpt-4-1106-preview"):
    client = OpenAI()
    chat_completion = client.chat.completions.create(
        messages=[
            {
                "role": "user",
                "content": prompt,
            }
        ],
        model=model,
    )
    return chat_completion.choices[0].message.content



from setuptools import find_packages, setup

setup(
    name="turbo_docs",
    version="1.1.1",
    packages=find_packages(),
    install_requires=[
        "openai>=1.3.6",
        "click",
        "pyperclip",
        "toml",
        "pathspec",
        "tiktoken",
    ],
    entry_points={
        "console_scripts": ["turbo_docs=turbo_docs.generate:driver"],
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
)

# python setup.py sdist bdist_wheel
# twine upload dist/*
